title = 'WordFreq v1'

import sys, re, os, glob
import Tkinter, tkFileDialog, tkMessageBox
from Tkinter import *

class WordFreq:
    def __init__(self, parent):
        self.parent = parent
        self.ResetWordlist()
	
	# Create the UI controls
	self.CreateStartDirectory()
	self.CreateFileSearch()
	self.CreateFileList()
	self.CreateWordfilter()
		
	self.SetDefaultValues()
	
	self.UpdateUI()
	
    def SetDefaultValues(self):
	self.strVarDir.set('c:\\temp')
	self.strVarFile.set('*.txt')
	self.strVarFilter.set('\/()[]!"#%&=?+-{},*\'.:\'#&%<>;$@|')	
	
    def CreateWordfilter(self):
	# Create Filter label and input field
	self.lblFilter = Tkinter.Label(self.parent,
#				       underline = 0,
				       text = "Character filter:")
	self.lblFilter.grid(row=3,column=0, sticky=W, padx=5, pady=5)
	
	self.strVarFilter = StringVar()
#	self.strVarFilter.trace("w", self.OnWordFilterChanged)
	
	self.entryFldFilter = Tkinter.Entry(self.parent, 
					    textvariable = self.strVarFilter,
					    )
	self.entryFldFilter.grid(row=3,column=1, sticky=W+E+N+S, padx=5, pady=5)

	self.btnExecute = Tkinter.Button(self.parent, width = 9,
					 text = 'Execute',
					 command = self.DoExecute)
	self.btnExecute.grid(row=3,column=4, sticky=E+N+S, padx=5, pady=5)
	

    def CreateFileList(self):
	# Create the ListBox and related controls
	self.lblFiles = Tkinter.Label(self.parent, text = "File List:")
	self.lblFiles.grid(row=2,column=0, sticky=W, padx=5, pady=5)
	
	self.lbScrollbar = Scrollbar(self.parent, orient=VERTICAL)
	self.lbFiles = Tkinter.Listbox(self.parent,
				       selectmode = 'multiple',
				       width = 60,
				       height = 6,
				       )
	self.lbFiles.grid(row=2,column=1, sticky=W+E+N+S, padx=5, pady=5)			    
	self.lbScrollbar.config(command=self.lbFiles.yview)
	self.lbScrollbar.grid(row=2,column=2, sticky=W+E+N+S)
	
    def CreateFileSearch(self):
	# Create File Search related controls
	self.lblFileSearch = Tkinter.Label(self.parent, text = "File pattern:")
	self.lblFileSearch.grid(row=1,column=0, sticky=W, padx=5, pady=5)
	
	self.strVarFile = StringVar()
	self.strVarFile.trace("w", self.OnFileSearchChanged)
	
	self.entryFldFileSearch = Tkinter.Entry(self.parent, 
						textvariable = self.strVarFile)
	self.entryFldFileSearch.grid(row=1,column=1,
				     sticky=W+E+N+S, padx=5, pady=5)

	self.btnSearch = Tkinter.Button(self.parent, width = 9,
					text = 'Search',
					command = self.DoSearch)
	self.btnSearch.grid(row=1,column=4, sticky=E+N+S, padx=5, pady=5)	
	
    def CreateStartDirectory(self):
	# Create Start Directory related controls	
	self.lblStartDir = Tkinter.Label(self.parent, text = "Start Directory:")
	self.lblStartDir.grid(row=0,column=0, sticky=W, padx=5, pady=5)

	self.strVarDir = StringVar()
	self.strVarDir.trace("w", self.OnStartDirChanged)
	
	self.entryFldStartDir = Tkinter.Entry(self.parent,
					      textvariable = self.strVarDir)
	self.entryFldStartDir.grid(row=0,column=1, sticky=W+E+N+S, padx=5, pady=5)

	self.btnBrowse = Tkinter.Button(self.parent, width = 9,
					text = 'Browse...',
					command = self.DoBrowseStartDir)
	self.btnBrowse.grid(row=0,column=4, sticky=E+N+S, padx=5, pady=5)	

    def OnFileSearchChanged(self, *args):
	search_pattern = self.strVarFile.get()
        if(len(search_pattern) > 0):
            self.btnSearch.configure(state='normal')
        else:
            self.btnSearch.configure(state='disabled')

#    def OnWordFilterChanged(self, *args):
#	print "Word filter changed"

    def OnStartDirChanged(self,*args):
        dirname = self.strVarDir.get()
        if(os.path.isdir(dirname)):
            self.btnSearch.configure(state='normal')
        else:
            self.btnSearch.configure(state='disabled')
	
		
    # Function for keeping buttons disabled if entry fields are not correct
    def UpdateUI(self):
        if(self.lbFiles.size() < 1):
	    self.btnExecute.configure(state='disabled')
        else:
            self.btnExecute.configure(state='normal')
	if(len(self.entryFldFileSearch.get()) < 1 ):
	    self.btnSearch.configure(state='disabled')
	else:
	    self.btnSearch.configure(state='normal')

    def ResetWordlist(self):
        self.wordlist = dict({})
       
    def DoBrowseStartDir(self):        
        dirname = tkFileDialog.askdirectory(parent=self.parent,
                                            initialdir="/",
                                            title='Select a directory')
        if len(dirname ) > 0 and os.path.isdir(dirname):
	    self.entryFldStartDir.delete(0,END)
            self.entryFldStartDir.insert(0,dirname)
        
        self.UpdateUI()

    def DoExecute(self):
        filenames = self.lbFiles.get(0,END)
        
        if(len(filenames) > 0 ):
            for filename in filenames:
                self.open_file(filename)
        
        sorted_words = self.wordlist.keys()        
        sorted_words.sort()
        
        amount_words = len(sorted_words)
        
	if tkMessageBox.askyesno("WordFreq: Result",
				 str(amount_words) + " unique words found.\n"+
				 "Save this result?"):
	    self.DoSaveWordList()

        self.UpdateUI()              

    def DoSaveWordList(self):
	file_types = [
	    ('Text File','*.txt'),
            ('Any File','*.*')]
	
        filename = tkFileDialog.asksaveasfilename(parent=self.parent,
                                            initialdir="/",
					    filetypes=file_types,
					    initialfile="wordlist_result",
					    defaultextension=".txt",
                                            title='Save wordlist result As')
        
        if len(filename) > 0:
            try:
		with open(filename,'w') as f:
		    sorted_words = self.wordlist.keys()
		    sorted_words.sort()
		    for word in sorted_words:
			f.write(str(word) +
				'\t' + str(self.wordlist[word]) +
				'\n')
	    except IOError:
		exctype, value = sys.exc_info()[:2]
		
		tkMessageBox.showerror("WordFreq Error",value)
                raise
      
    def DoSearch(self):
        startdir = self.entryFldStartDir.get()
        filename = self.entryFldFileSearch.get()

        file_list = self.get_file_list(startdir, filename)
        found_files = file_list.keys()
	found_files.sort()
	
        self.lbFiles.delete(0,END)
        
	for file in found_files:
	    self.lbFiles.insert(END,file)
       
        self.UpdateUI()

    def clean_up_line(self,line):
        
        filter = re.escape(self.entryFldFilter.get())
	
	if(len(filter) > 0):
	    line = re.sub("["+str(filter)+"]+",' ',line)
	    line = re.sub("[ ]+",' ',line)
	else:
	    line = re.sub("[ ]+",' ',line)
	    
        return line
    
    def add_word_to_wordlist(self,line):
        clean_line = self.clean_up_line(line)
        words = clean_line.split()
        
        for word in words:
            if(word in self.wordlist):
                n = self.wordlist[word]
                self.wordlist[word] = n+1
            else:
                self.wordlist[word] = 1
    
    def open_file(self,filename):
        if(os.path.isfile(filename)):
	    try:
		with open(filename, 'r') as f:
		    for line in f:
			self.add_word_to_wordlist(line)
	    except IOError:
		exctype, value = sys.exc_info()[:2]
		
		tkMessageBox.showerror("WordFreq Error",value)
                raise
         
    def get_file_list(self, startdir, filename):
        found_files = ({})
	    
	try:        
	    for root, dirs, files in os.walk(startdir):
		thefiles = glob.glob(os.path.abspath(root)+"\\"+filename)
		
		for thefile in thefiles:
		    if(os.path.isfile(thefile)):
			found_files[thefile] = 1
	except:
	    exctype, value = sys.exc_info()[:2]
	    
	    tkMessageBox.showerror("WordFreq Error",value)
	    raise
	    
        return found_files

######################################################################

# Create root window
if __name__ == '__main__':
    root = Tkinter.Tk()
    root.title(title)
    root.wm_iconbitmap(bitmap="icon.ico")
    root.resizable(width=0, height=0)

    exitButton = Tkinter.Button(root,
				width=9,
				text = 'Exit',
				command = root.destroy)
    exitButton.grid(row=5, column=4, padx=5, pady=5)
    widget = WordFreq(root)
    root.mainloop()

